unit InfoDialog;

interface

uses Windows, Messages, ShellAPI, NTypes, NLib, NForm, NkvImage;

type
  TInfoDlg = class(TModelessDlg)
    Path: string;
    PathInArchive: string;
    Image: TNkVImage;
    Icon: HICON;
    ATime, MTime: Int64;
    procedure UpdateInfo;
    procedure WMDrawItem(var Message: TWMDrawItem); message WM_DRAWITEM;
    procedure WMInitDialog(var Message: TWMInitDialog); message WM_INITDIALOG;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND; 
  end;

implementation

{$I dialogid.inc}

function GetIcon(const Path: string): HICON;
var
  id: Word;
begin
  id := 1; // if id=1, It is not thumbnail icon but normal file icon
  Result := ExtractAssociatedIcon(HInstance, PChar(Path), id);
end;

function BytesFormat(Value: Integer): string;
var
  s: string;
begin
  Result := UnitFormat(Value);
  s := Copy(Result, 1, Length(Result)-1);
  case Result[Length(Result)] of
    'G': Result := s + ' GB';
    'M': Result := s + ' MB';
    'K': Result := s + ' KB';
  else
    Result := Result + ' bytes';
  end;
end;

function NumFormat(Value: Integer): string;
var
  nf: TNumberFmt;
  buf: array[0..255] of Char;
begin
  nf.NumDigits := 0;
  nf.LeadingZero := 0;
  nf.Grouping := 3;
  nf.lpDecimalSep := '.';
  nf.lpThousandSep := ',';
  nf.NegativeOrder := 1;
  GetNumberFormat(0, 0, PChar(IStr(Value)), @nf, buf, SizeOf(buf));
  Result := buf;
end;

procedure TInfoDlg.UpdateInfo;
var
  t: TTime;
begin
  if Path = '' then
  begin
    Icon := 0;
    Items[IDC_FILENAME].SetText('<STDIN>')
  end else
  begin
    Icon := GetIcon(Path);
    if PathInArchive = '' then
      Items[IDC_FILENAME].SetText(BaseName(Path))
    else
      Items[IDC_FILENAME].SetText(PathInArchive)
  end;
  if PathInArchive = '' then
    Items[IDC_LOCATION].SetText(DirName(Path))
  else
    Items[IDC_LOCATION].SetText(Path);
  Items[IDC_SIZE].SetText(Fmt('{0}x{1}', [Image.Width, Image.Height]));
  Items[IDC_BPP].SetText(Fmt('{0} bits/pixel', [Image.Bpp]));
  Items[IDC_FORMAT].SetText(Image.FormatName);
  Items[IDC_FILESIZE].SetText(Fmt('{0} ({1} bytes)',
      [BytesFormat(Image.OrgSize), NumFormat(Image.OrgSize)]));
  t := LocalTime(MTime);
  Items[IDC_MODIFIED].SetText(Fmt('{0:04}-{1:02}-{2:02} {3:02}:{4:02}:{5:02}',
      [t.Year, t.Mon, t.MDay, t.Hour, t.Min, t.Sec]));
  if PathInArchive = '' then
  begin
    t := LocalTime(ATime);
    Items[IDC_ACCESSED].SetText(Fmt('{0:04}-{1:02}-{2:02} {3:02}:{4:02}:{5:02}',
        [t.Year, t.Mon, t.MDay, t.Hour, t.Min, t.Sec]));
  end else
    Items[IDC_ACCESSED].SetText('');
  Items[IDC_MEMORY].SetText(Fmt('{0} ({1} bytes)', [BytesFormat(Image.BitsSize),
    NumFormat(Image.BitsSize)]));
  Items[IDC_PLUGIN].SetText(Image.FilterName);
  Items[IDC_PLUGINFILE].SetText(Image.FilterFilename);
  Items[IDC_TEXTINFOLABEL].Enable(Image.Comment <> '');
  Items[IDC_TEXTINFO].SetText(Image.Comment);
  InvalidateRect(Items[IDC_ICON].Handle, nil, False);
end;

procedure TInfoDlg.WMDrawItem(var Message: TWMDrawItem);
begin
  with Message do
    if Ctl = IDC_ICON  then
      with DrawItemStruct^ do
      begin
        FillRect(hDC, rcItem, GetSysColorBrush(COLOR_3DFACE));
        DrawIcon(hDC, 0, 0, Icon);
        Result := 1;
      end;
end;

procedure TInfoDlg.WMInitDialog(var Message: TWMInitDialog);
begin
  UpdateInfo;
  Message.Result := 0;
end;

procedure TInfoDlg.WMCommand(var Message: TWMCommand);
begin
  with Message do
    case ItemID of
      IDOK, IDCANCEL:
        begin
          Result := 1;
          Close;
        end;
    end;
end;

end.

